/*  Copyright 2009 Marc Toussaint
    email: mtoussai@cs.tu-berlin.de

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a COPYING file of the GNU General Public License
    along with this program. If not, see <http://www.gnu.org/licenses/> */

#include "soc.h"
#include "opengl.h"


void soc::getQuadraticTaskCost(soc::SocAbstraction& soci,int t,arr& R,arr& r,const arr& qt){
  uint m=soci.nTasks(),n=soci.qDim();
  uint i;
  arr phiHatQ,J,tJ,x;
  double prec;
  R.resize(n,n); R.setZero();
  r.resize(n);   r.setZero();
  for(i=0;i<m;i++) if(soci.isConditioned(i,t)){
    soci.getPhi      (phiHatQ,i);
    soci.getTarget   (x,prec,i,t);
    soci.getJtJ      (J,tJ,i);
    R += prec*tJ*J;
    r -= 2.*prec*tJ*(x - phiHatQ + J*qt);
  }
}



double getTaskLogLikelihood(soc::SocAbstraction& soci,int t){
  uint m=soci.nTasks();
  uint i;
  double L=0.;
  arr phiHatQ,Jqd,x,v;
  double prec;
  for(i=0;i<m;i++) if(soci.isConditioned(i,t)){
    soci.getPhi(phiHatQ,i);
    soci.getTarget (x,prec,i,t);
    NIY;
    //L += logNNprec(x,phiHatQ,prec);
    if(soci.dynamic){
      soci.getJqd(Jqd,i);
      soci.getTargetV(v,prec,i,t);
      //L += logNNprec(v,Jqd,prec);
    }
  }
  return L;
}

double getFilterCostMeassure(soc::SocAbstraction& soci,arr& q,double& cost1,double& cost2,std::ostream *os){
  //evaluate trajectory
  double cost_t,length=0.;
  cost1=.0;
  cost2=.0;
  double tau=soci.getTau(),tau2=tau*tau;
  arr v;
  arr W,H,invH,Q,invQ,Minv,F,tmp1,tmp2,tmp3,tmp4,d;
  soci.getW(W);
  soci.getH(H);
  soci.getQ(Q);
  inverse_SymPosDef(invQ,Q);
  inverse_SymPosDef(invH,H);
  uint t,T=q.d0;
  for(t=0;t<T;t++){
    soc::getVelocity(v,q,t,tau);
    soci.setqv(q[t],v);
    if(t>1){
      soci.getMinvF(Minv,F);
      tmp1 = 2.*q[t-1]+q[t-2]+tau2*Minv*F - q[t];
      cost2 += scalarProduct(invQ,tmp1,tmp1);
    }
    if(t==1){
      tmp1 = q[t-1] - q[t];
      cost2 += scalarProduct(invQ,tmp1,tmp1);
    }
    if(t>0){ d=q[t]-q[t-1]; length += ::sqrt(scalarProduct(W,d,d)); }
    CHECK(soci.dynamic==false,"");
    cost1 += cost_t = getTaskCost(soci,t);
  }
  if(os){
    *os <<std::setw(3) <<0
        <<"  time " <<MT::timerRead(false)
        <<"  cost1 " <<cost1
        <<"  cost2 " <<cost2
        <<"  length " <<length
        <<"  total-cost " <<cost1+cost2 <<endl;
  }
  return cost1+cost2;
}

